﻿
// The sole owner of this code is Dundas Data Visualization Inc.
// Copyright (c) 2015 Dundas Data Visualization Inc.
// All rights reserved.

#region Header

//using DocumentFormat.OpenXml;
//using DocumentFormat.OpenXml.Packaging;
//using DocumentFormat.OpenXml.Spreadsheet;

#endregion Header

using Dundas.BI;
using Dundas.BI.Configuration;
using Dundas.BI.Data.DataRetrieval;
using Dundas.BI.Export;
using Dundas.BI.Extensibility;

using PdfSharp.Pdf;
using PdfSharp.Pdf.IO;
using PdfSharp.Pdf.Security;

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using System.Threading.Tasks;

namespace ServerEventsSample
{
	/// <summary>
	/// This class represents the extension package info for the Server Events. 
	/// </summary>
	public class ExportCompletedExtensionPackageInfo : ExtensionPackageInfo
	{
		#region Static Fields

		internal static readonly Guid ModuleId = new Guid("4befb94d-80d1-4dc0-9d06-7cb16953a0de");
		public static Guid PdfUserPasswordSettingId = new Guid("6a66a97c-89cb-4720-a5df-4806cc239073");
		public static Guid PdfOwnerPasswordSettingId = new Guid("7fb599c1-a975-4db2-9896-6b1bcc6d97a4");

		#endregion Static Fields

		#region Public Properties

		/// <summary>
		/// Gets the name of the extension package author.
		/// </summary>
		public override string Author
		{
			get { return "Dundas Data Visualization Sample Author"; }
		}

		/// <summary>
		/// Gets the copyright text associated with the extension package.
		/// </summary>
		public override string Copyright
		{
			get { return "Dundas Data Visualization, Inc."; }
		}

		public override string DisplayName
		{
			get { return "Export Completed Sample"; }
		}

		public override Guid Id
		{
			get { return ServerEventsSample.ExportCompletedExtensionPackageInfo.ModuleId; }
		}

		public override string Name
		{
			get { return "Export Completed Sample"; }
		}

		/// <summary>
		/// Gets the version of the extension package.
		/// </summary>
		public override Version Version
		{
			get { return new Version(0, 0, 1);  }
		}

		#endregion Public Properties

		#region Private Methods

        private void CreatePasswordSettings()
        {
	        // Get the application configuration service.
	        IAppConfigService appConfigService = Engine.Current.GetService<IAppConfigService>();

	        // Create the properties for the user and owner passwords for the PDF document.

	        AppSettingProperties appSettingProperties;

	        appSettingProperties = new AppSettingProperties(
		        PdfUserPasswordSettingId,
		        "PdfUserPassword",
		        ExportCompletedExtensionPackageInfo.ModuleId,
		        typeof(string)
		        );

	        appSettingProperties.CategoryName = "CustomExtension";
	        appSettingProperties.IsEncrypted = true;
	        appSettingProperties.IsPassword = true;
	        appSettingProperties.DefaultValue = "1234";

	        appConfigService.RegisterSetting(appSettingProperties);

	        appSettingProperties = new AppSettingProperties(
		        PdfOwnerPasswordSettingId,
		        "PdfOwnerPassword",
		        ExportCompletedExtensionPackageInfo.ModuleId,
		        typeof(string)
		        );
	        appSettingProperties.CategoryName = "CustomExtension";
	        appSettingProperties.IsEncrypted = true;
	        appSettingProperties.IsPassword = true;
	        appSettingProperties.DefaultValue = "1234";

	        appConfigService.RegisterSetting(appSettingProperties);
        }

        private void eventHookService_ExportCompleted(object sender, Dundas.BI.Export.ExportCompletedEventArgs e)
        {
	        ExportResult result = e.ExportResult;

	        if (e.ExportRequest.ProviderId.Equals(Dundas.BI.Export.StandardExportProviders.StandardExportProviderIds.Pdf))
	        {
		        // file is stored in
		        if (e.ExportResult.Content == null && result.BlobId.HasValue)
		        {
			        IExportService exportService = Engine.Current.GetService<IExportService>();
			        result = exportService.GetExportBlobResult(result.BlobId.Value);
		        }

		        // write the file back to disk to be able to manipulate it
		        string tempFilePath =
			        Path.Combine(
				        Path.GetTempPath(),
				        Guid.NewGuid() + (".pdf")
			        );

		        using (FileStream tempFile = File.Create(tempFilePath))
		        {
			        result.Content.CopyTo(tempFile);
		        }


		        // Open an existing document.
		        PdfDocument document = PdfReader.Open(tempFilePath);

		        PdfSecuritySettings securitySettings = document.SecuritySettings;

		        // Get the application configuration service.
		        IAppConfigService appConfigService = Engine.Current.GetService<IAppConfigService>();

		        // Setting one of the passwords automatically sets the security level to PdfDocumentSecurityLevel.Encrypted128Bit.
		        securitySettings.UserPassword = appConfigService.GetString(PdfUserPasswordSettingId);
		        securitySettings.OwnerPassword = appConfigService.GetString(PdfOwnerPasswordSettingId);

		        // Don't use 40 bit encryption unless needed for compatibility
		        //securitySettings.DocumentSecurityLevel = PdfDocumentSecurityLevel.Encrypted40Bit;

		        // Restrict some rights.
		        securitySettings.PermitAccessibilityExtractContent = false;
		        securitySettings.PermitAnnotations = false;
		        securitySettings.PermitAssembleDocument = false;
		        securitySettings.PermitExtractContent = false;
		        securitySettings.PermitFormsFill = true;
		        securitySettings.PermitFullQualityPrint = true;
		        securitySettings.PermitModifyDocument = false;
		        securitySettings.PermitPrint = true;

		        // Save the document...
		        document.Save(tempFilePath);

		        // Set the UpdatedExportContentFileInfo to the new temp file path.
		        e.UpdatedExportContentFileInfo = new System.IO.FileInfo(tempFilePath);
	        }
        }

		#endregion Private Methods

		#region Public Methods

        /// <summary>
        /// Called after the extension package is loaded during engine startup.
        /// </summary>
        public override void OnLoaded()
        {
	        // Create application settings to store the passwords.
	        CreatePasswordSettings();

	        IEventHookService eventHookService =
		        Engine.Current.GetService<IEventHookService>();

	        // Hooks up event handler when a file system entry is created.
	        eventHookService.ExportCompleted += eventHookService_ExportCompleted;
        }

		#endregion Public Methods
	}
}